@@ -113,41 +113,36 @@ class VideoAdminTestCase(VideoBaseTestCase, AdminTests, AdminBaseTestCase):
     def test_validate_allowed_asset_type(self):
         """ If asset type is not allowed, object will not be saved."""
         self.asset_type.required_for.clear()
-        r = self.client.get(self.change_url)
-        data = self.get_form_data_from_response(r)
-        url = self.add_url
-        data = self.transform_to_new(data)
-        data[f'{self.prefix}-0-image'] = self.create_uploaded_image(self.image)
-
-        r = self.client.post(url, data=data)
+        kw = {f'{self.prefix}-0-image': self.create_uploaded_image(self.image)}
+        r = self.post_changeform(fields=kw)
 
         self.assertEqual(r.status_code, 200)
         self.assertIsNotNone(self.get_errors_from_response(r))
         self.set_allowed_for(self.asset_type, models.Video)
 
-        data[f'{self.prefix}-0-image'] = self.create_uploaded_image(self.image)
-
-        r = self.client.post(url, data=data)
+        kw = {f'{self.prefix}-0-image': self.create_uploaded_image(self.image)}
+        r = self.post_changeform(fields=kw)
 
         self.assertFalse(self.get_errors_from_response(r))
         self.assertEqual(r.status_code, 302)
 
     def test_maintain_single_active_asset(self):
         """ Only one active asset of same asset type is allowed."""
-        r = self.client.get(self.change_url)
-        data = self.get_form_data_from_response(r)
-        data[f'{self.prefix}-1-image'] = self.create_uploaded_image(self.image)
-        data[f'{self.prefix}-1-asset_type'] = self.asset_type.id
-
-        r = self.client.post(self.change_url, data=data)
+        kw = {
+            f'{self.prefix}-1-image': self.create_uploaded_image(self.image),
+            f'{self.prefix}-1-asset_type': self.asset_type.id,
+        }
+        r = self.post_changeform(fields=kw)
 
         self.assertEqual(r.status_code, 200)
         self.assertTrue(self.get_errors_from_response(r))
 
-        data[f'{self.prefix}-0-active'] = False
-        data[f'{self.prefix}-1-image'] = self.create_uploaded_image(self.image)
-
-        r = self.client.post(self.change_url, data=data)
+        kw = {
+            f'{self.prefix}-0-active': False,
+            f'{self.prefix}-1-image': self.create_uploaded_image(self.image),
+            f'{self.prefix}-1-asset_type': self.asset_type.id,
+        }
+        r = self.post_changeform(fields=kw)
 
         self.assertFalse(self.get_errors_from_response(r))
         self.assertEqual(r.status_code, 302)
@@ -157,6 +152,35 @@ class VideoAdminTestCase(VideoBaseTestCase, AdminTests, AdminBaseTestCase):
         asset = self.video.assets.last()
         self.assertTrue(asset.active)
 
+    def test_multiple_inactive_assets_allowed(self):
+        """
+        There may be multiple inactive assets of same type for same object.
+        :return:
+        """
+        kw = {
+            f'{self.prefix}-1-active': False,
+            f'{self.prefix}-1-image': self.create_uploaded_image(self.image),
+            f'{self.prefix}-1-asset_type': self.asset_type.id,
+            f'{self.prefix}-2-active': False,
+            f'{self.prefix}-2-image': self.create_uploaded_image(self.image),
+            f'{self.prefix}-2-asset_type': self.asset_type.id,
+        }
+        r = self.post_changeform(fields=kw)
+
+        self.assertFalse(self.get_errors_from_response(r))
+        self.assertEqual(r.status_code, 302)
+        self.assertEqual(2, self.video.assets.filter(active=False).count())
+
+
+class AssetModelTestCase(VideoBaseTestCase):
+    """ Asset model test case"""
+
+    def test_str(self):
+        """ Check __str__ method."""
+        self.assertIsInstance(self.asset.__str__(), str)
+        empty = assets_models.get_asset_model()()
+        self.assertIsInstance(empty.__str__(), str)
+
 
 class DeletedAssetModelTestCase(VideoBaseTestCase):
     """ Asset deletion handling test case."""
@@ -171,6 +195,15 @@ class DeletedAssetModelTestCase(VideoBaseTestCase):
         super().tearDown()
         self.delete_patcher.stop()
 
+    def test_str(self):
+        """ Check __str__ method."""
+        self.asset.delete()
+        deleted_model = assets_models.get_deleted_asset_model()
+        deleted = deleted_model.objects.last()
+        self.assertIsInstance(deleted.__str__(), str)
+        empty = deleted_model()
+        self.assertIsInstance(empty.__str__(), str)
+
     def test_delete_asset(self):
         """
         Asset file is moved to DeletedAsset instance after asset deletion.
@@ -194,17 +227,53 @@ class DeletedAssetModelTestCase(VideoBaseTestCase):
 
         self.delete_mock.assert_called_once_with(filename)
 
+    def test_delete_multiple_assets_for_same_object(self):
+        """
+        There may be multiple deleted assets of same type for same object.
+        """
+        self.asset.delete()
+        asset = self.create_asset(self.asset.asset_type, self.asset.related)
+        asset.delete()
+
+        deleted = assets_models.DeletedAsset.objects.filter(
+            content_type=self.asset.content_type,
+            object_id=self.asset.object_id,
+            asset_type=self.asset.asset_type)
+        self.assertEqual(2, deleted.count())
+
+    def test_recover_deleted_asset(self):
+        """
+        Deleted asset is recovered as inactive asset with same image.
+        """
+        self.asset.delete()
+        deleted = assets_models.DeletedAsset.objects.get()
+        qs = assets_models.DeletedAsset.objects.filter(pk=deleted.pk)
+        filename = deleted.image.name
+
+        deleted.recover()
+
+        asset = assets_models.Asset.objects.order_by('id').last()
+        self.assert_object_fields(
+            asset,
+            content_type=self.asset.content_type,
+            object_id=self.asset.object_id,
+            asset_type=self.asset.asset_type,
+            active=False,
+        )
+        self.assertEqual(asset.image.name, filename)
+        self.delete_mock.assert_not_called()
+        self.assertFalse(qs.exists())
+
 
 class AssetValidationTestCase(VideoBaseTestCase):
     """ Checks image validation for asset type."""
 
     def assert_validation_not_passed(self):
-        try:
+        with self.assertRaises(ValidationError) as e:
             self.asset.full_clean()
-        except ValidationError as e:
-            invalid_keys = set(e.error_dict)
-            expected = {'asset_type', 'content_type', 'object_id'}
-            self.assertTrue(invalid_keys - expected)
+        invalid_keys = set(e.exception.error_dict)
+        expected = {'asset_type', 'content_type', 'object_id'}
+        self.assertTrue(invalid_keys - expected)
 
     def assert_validation_passed(self):
         try:
